# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .available_tool import AvailableTool


class ToolCategory(UniversalBaseModel):
    """
    将可用工具按功能分类
    """

    category: str = pydantic.Field()
    """
    Name of tool category
    """

    tools: typing.List[AvailableTool] = pydantic.Field()
    """
    List of tools under this category
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

from ..core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ..core.request_options import RequestOptions
from ..types.response import Response
from ..types.response_list_sandbox_hook import ResponseListSandboxHook
from ..types.response_sandbox_hook import ResponseSandboxHook
from ..types.sandbox_response import SandboxResponse
from .raw_client import AsyncRawSandboxClient, RawSandboxClient

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class SandboxClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._raw_client = RawSandboxClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> RawSandboxClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        RawSandboxClient
        """
        return self._raw_client

    def get_context(self, *, request_options: typing.Optional[RequestOptions] = None) -> SandboxResponse:
        """
        Get sandbox environment information

        Parameters
        ----------
        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        SandboxResponse
            Successful Response

        Examples
        --------
        from agent_sandbox import Sandbox

        client = Sandbox(
            base_url="https://yourhost.com/path/to/api",
        )
        client.sandbox.get_context()
        """
        _response = self._raw_client.get_context(request_options=request_options)
        return _response.data

    def get_python_packages(self, *, request_options: typing.Optional[RequestOptions] = None) -> Response:
        """
        Get installed Python packages

        Parameters
        ----------
        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        Response
            Successful Response

        Examples
        --------
        from agent_sandbox import Sandbox

        client = Sandbox(
            base_url="https://yourhost.com/path/to/api",
        )
        client.sandbox.get_python_packages()
        """
        _response = self._raw_client.get_python_packages(request_options=request_options)
        return _response.data

    def get_nodejs_packages(self, *, request_options: typing.Optional[RequestOptions] = None) -> Response:
        """
        Get installed Node.js packages

        Parameters
        ----------
        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        Response
            Successful Response

        Examples
        --------
        from agent_sandbox import Sandbox

        client = Sandbox(
            base_url="https://yourhost.com/path/to/api",
        )
        client.sandbox.get_nodejs_packages()
        """
        _response = self._raw_client.get_nodejs_packages(request_options=request_options)
        return _response.data

    def list_hooks(
        self, *, event: typing.Optional[str] = None, request_options: typing.Optional[RequestOptions] = None
    ) -> ResponseListSandboxHook:
        """
        List registered lifecycle hooks, optionally filtered by event.

        Parameters
        ----------
        event : typing.Optional[str]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ResponseListSandboxHook
            Successful Response

        Examples
        --------
        from agent_sandbox import Sandbox

        client = Sandbox(
            base_url="https://yourhost.com/path/to/api",
        )
        client.sandbox.list_hooks(
            event="event",
        )
        """
        _response = self._raw_client.list_hooks(event=event, request_options=request_options)
        return _response.data

    def register_hook(
        self,
        *,
        name: str,
        command: str,
        event: typing.Optional[str] = OMIT,
        timeout: typing.Optional[float] = OMIT,
        priority: typing.Optional[int] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> ResponseSandboxHook:
        """
        Register a lifecycle hook. Currently supported events: shutdown.

        Parameters
        ----------
        name : str
            Unique name for this hook

        command : str
            Shell command to execute

        event : typing.Optional[str]
            Lifecycle event: "shutdown"

        timeout : typing.Optional[float]
            Per-hook timeout in seconds

        priority : typing.Optional[int]
            Execution priority (lower = earlier). Same priority hooks run in parallel

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ResponseSandboxHook
            Successful Response

        Examples
        --------
        from agent_sandbox import Sandbox

        client = Sandbox(
            base_url="https://yourhost.com/path/to/api",
        )
        client.sandbox.register_hook(
            name="name",
            command="command",
        )
        """
        _response = self._raw_client.register_hook(
            name=name, command=command, event=event, timeout=timeout, priority=priority, request_options=request_options
        )
        return _response.data

    def remove_hook(self, name: str, *, request_options: typing.Optional[RequestOptions] = None) -> Response:
        """
        Remove a hook by name. ENV hooks cannot be removed.

        Parameters
        ----------
        name : str

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        Response
            Successful Response

        Examples
        --------
        from agent_sandbox import Sandbox

        client = Sandbox(
            base_url="https://yourhost.com/path/to/api",
        )
        client.sandbox.remove_hook(
            name="name",
        )
        """
        _response = self._raw_client.remove_hook(name, request_options=request_options)
        return _response.data


class AsyncSandboxClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._raw_client = AsyncRawSandboxClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> AsyncRawSandboxClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        AsyncRawSandboxClient
        """
        return self._raw_client

    async def get_context(self, *, request_options: typing.Optional[RequestOptions] = None) -> SandboxResponse:
        """
        Get sandbox environment information

        Parameters
        ----------
        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        SandboxResponse
            Successful Response

        Examples
        --------
        import asyncio

        from agent_sandbox import AsyncSandbox

        client = AsyncSandbox(
            base_url="https://yourhost.com/path/to/api",
        )


        async def main() -> None:
            await client.sandbox.get_context()


        asyncio.run(main())
        """
        _response = await self._raw_client.get_context(request_options=request_options)
        return _response.data

    async def get_python_packages(self, *, request_options: typing.Optional[RequestOptions] = None) -> Response:
        """
        Get installed Python packages

        Parameters
        ----------
        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        Response
            Successful Response

        Examples
        --------
        import asyncio

        from agent_sandbox import AsyncSandbox

        client = AsyncSandbox(
            base_url="https://yourhost.com/path/to/api",
        )


        async def main() -> None:
            await client.sandbox.get_python_packages()


        asyncio.run(main())
        """
        _response = await self._raw_client.get_python_packages(request_options=request_options)
        return _response.data

    async def get_nodejs_packages(self, *, request_options: typing.Optional[RequestOptions] = None) -> Response:
        """
        Get installed Node.js packages

        Parameters
        ----------
        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        Response
            Successful Response

        Examples
        --------
        import asyncio

        from agent_sandbox import AsyncSandbox

        client = AsyncSandbox(
            base_url="https://yourhost.com/path/to/api",
        )


        async def main() -> None:
            await client.sandbox.get_nodejs_packages()


        asyncio.run(main())
        """
        _response = await self._raw_client.get_nodejs_packages(request_options=request_options)
        return _response.data

    async def list_hooks(
        self, *, event: typing.Optional[str] = None, request_options: typing.Optional[RequestOptions] = None
    ) -> ResponseListSandboxHook:
        """
        List registered lifecycle hooks, optionally filtered by event.

        Parameters
        ----------
        event : typing.Optional[str]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ResponseListSandboxHook
            Successful Response

        Examples
        --------
        import asyncio

        from agent_sandbox import AsyncSandbox

        client = AsyncSandbox(
            base_url="https://yourhost.com/path/to/api",
        )


        async def main() -> None:
            await client.sandbox.list_hooks(
                event="event",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.list_hooks(event=event, request_options=request_options)
        return _response.data

    async def register_hook(
        self,
        *,
        name: str,
        command: str,
        event: typing.Optional[str] = OMIT,
        timeout: typing.Optional[float] = OMIT,
        priority: typing.Optional[int] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> ResponseSandboxHook:
        """
        Register a lifecycle hook. Currently supported events: shutdown.

        Parameters
        ----------
        name : str
            Unique name for this hook

        command : str
            Shell command to execute

        event : typing.Optional[str]
            Lifecycle event: "shutdown"

        timeout : typing.Optional[float]
            Per-hook timeout in seconds

        priority : typing.Optional[int]
            Execution priority (lower = earlier). Same priority hooks run in parallel

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ResponseSandboxHook
            Successful Response

        Examples
        --------
        import asyncio

        from agent_sandbox import AsyncSandbox

        client = AsyncSandbox(
            base_url="https://yourhost.com/path/to/api",
        )


        async def main() -> None:
            await client.sandbox.register_hook(
                name="name",
                command="command",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.register_hook(
            name=name, command=command, event=event, timeout=timeout, priority=priority, request_options=request_options
        )
        return _response.data

    async def remove_hook(self, name: str, *, request_options: typing.Optional[RequestOptions] = None) -> Response:
        """
        Remove a hook by name. ENV hooks cannot be removed.

        Parameters
        ----------
        name : str

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        Response
            Successful Response

        Examples
        --------
        import asyncio

        from agent_sandbox import AsyncSandbox

        client = AsyncSandbox(
            base_url="https://yourhost.com/path/to/api",
        )


        async def main() -> None:
            await client.sandbox.remove_hook(
                name="name",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.remove_hook(name, request_options=request_options)
        return _response.data



# This file was auto-generated by Fern from our API Definition.

import typing
from json.decoder import JSONDecodeError

from ..core.api_error import ApiError
from ..core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ..core.http_response import AsyncHttpResponse, HttpResponse
from ..core.pydantic_utilities import parse_obj_as
from ..core.request_options import RequestOptions
from ..errors.unprocessable_entity_error import UnprocessableEntityError
from ..types.http_validation_error import HttpValidationError
from ..types.response_display_record_result import ResponseDisplayRecordResult
from .types.record_action import RecordAction

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class RawDisplayClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper

    def record(
        self,
        *,
        action: RecordAction,
        save_path: typing.Optional[str] = OMIT,
        fps: typing.Optional[int] = OMIT,
        crf: typing.Optional[int] = OMIT,
        max_duration: typing.Optional[float] = OMIT,
        width: typing.Optional[int] = OMIT,
        height: typing.Optional[int] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[ResponseDisplayRecordResult]:
        """
        Control display screen recording (start/stop/status).

        Records the entire X11 desktop using ffmpeg x11grab,
        including browser UI, multiple tabs, popups, terminal, etc.

        Parameters
        ----------
        action : RecordAction
            Recording action: start, stop, or status

        save_path : typing.Optional[str]
            Output file path (default: /tmp/recordings/recording_{timestamp}.mp4)

        fps : typing.Optional[int]
            Frames per second

        crf : typing.Optional[int]
            H.264 CRF quality (0=lossless, 51=worst)

        max_duration : typing.Optional[float]
            Max recording duration in seconds

        width : typing.Optional[int]
            Video width in pixels (auto-detected from X11 if omitted)

        height : typing.Optional[int]
            Video height in pixels (auto-detected from X11 if omitted)

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[ResponseDisplayRecordResult]
            Successful Response
        """
        _response = self._client_wrapper.httpx_client.request(
            "v1/display/record",
            method="POST",
            json={
                "action": action,
                "save_path": save_path,
                "fps": fps,
                "crf": crf,
                "max_duration": max_duration,
                "width": width,
                "height": height,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    ResponseDisplayRecordResult,
                    parse_obj_as(
                        type_=ResponseDisplayRecordResult,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        HttpValidationError,
                        parse_obj_as(
                            type_=HttpValidationError,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)


class AsyncRawDisplayClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper

    async def record(
        self,
        *,
        action: RecordAction,
        save_path: typing.Optional[str] = OMIT,
        fps: typing.Optional[int] = OMIT,
        crf: typing.Optional[int] = OMIT,
        max_duration: typing.Optional[float] = OMIT,
        width: typing.Optional[int] = OMIT,
        height: typing.Optional[int] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[ResponseDisplayRecordResult]:
        """
        Control display screen recording (start/stop/status).

        Records the entire X11 desktop using ffmpeg x11grab,
        including browser UI, multiple tabs, popups, terminal, etc.

        Parameters
        ----------
        action : RecordAction
            Recording action: start, stop, or status

        save_path : typing.Optional[str]
            Output file path (default: /tmp/recordings/recording_{timestamp}.mp4)

        fps : typing.Optional[int]
            Frames per second

        crf : typing.Optional[int]
            H.264 CRF quality (0=lossless, 51=worst)

        max_duration : typing.Optional[float]
            Max recording duration in seconds

        width : typing.Optional[int]
            Video width in pixels (auto-detected from X11 if omitted)

        height : typing.Optional[int]
            Video height in pixels (auto-detected from X11 if omitted)

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[ResponseDisplayRecordResult]
            Successful Response
        """
        _response = await self._client_wrapper.httpx_client.request(
            "v1/display/record",
            method="POST",
            json={
                "action": action,
                "save_path": save_path,
                "fps": fps,
                "crf": crf,
                "max_duration": max_duration,
                "width": width,
                "height": height,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    ResponseDisplayRecordResult,
                    parse_obj_as(
                        type_=ResponseDisplayRecordResult,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        HttpValidationError,
                        parse_obj_as(
                            type_=HttpValidationError,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel


class ShellSessionStats(UniversalBaseModel):
    """
    Shell session statistics model
    """

    total_sessions: int = pydantic.Field()
    """
    Total number of sessions
    """

    active_sessions: int = pydantic.Field()
    """
    Number of active sessions (used within last 5 minutes)
    """

    idle_sessions: int = pydantic.Field()
    """
    Number of idle sessions
    """

    max_sessions: int = pydantic.Field()
    """
    Maximum allowed sessions
    """

    session_timeout: int = pydantic.Field()
    """
    Session timeout in seconds
    """

    usage_ratio: float = pydantic.Field()
    """
    Session usage ratio (0.0 to 1.0)
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel


class SandboxHook(UniversalBaseModel):
    """
    A registered lifecycle hook.
    """

    name: str = pydantic.Field()
    """
    Unique name for this hook
    """

    event: typing.Optional[str] = pydantic.Field(default=None)
    """
    Lifecycle event: "shutdown"
    """

    command: str = pydantic.Field()
    """
    Shell command to execute
    """

    timeout: typing.Optional[float] = pydantic.Field(default=None)
    """
    Per-hook timeout in seconds
    """

    priority: typing.Optional[int] = pydantic.Field(default=None)
    """
    Execution priority (lower = earlier). Same priority hooks run in parallel
    """

    source: typing.Optional[str] = pydantic.Field(default=None)
    """
    Registration source: "env" or "api"
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

from ..core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ..core.request_options import RequestOptions
from ..types.response_display_record_result import ResponseDisplayRecordResult
from .raw_client import AsyncRawDisplayClient, RawDisplayClient
from .types.record_action import RecordAction

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class DisplayClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._raw_client = RawDisplayClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> RawDisplayClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        RawDisplayClient
        """
        return self._raw_client

    def record(
        self,
        *,
        action: RecordAction,
        save_path: typing.Optional[str] = OMIT,
        fps: typing.Optional[int] = OMIT,
        crf: typing.Optional[int] = OMIT,
        max_duration: typing.Optional[float] = OMIT,
        width: typing.Optional[int] = OMIT,
        height: typing.Optional[int] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> ResponseDisplayRecordResult:
        """
        Control display screen recording (start/stop/status).

        Records the entire X11 desktop using ffmpeg x11grab,
        including browser UI, multiple tabs, popups, terminal, etc.

        Parameters
        ----------
        action : RecordAction
            Recording action: start, stop, or status

        save_path : typing.Optional[str]
            Output file path (default: /tmp/recordings/recording_{timestamp}.mp4)

        fps : typing.Optional[int]
            Frames per second

        crf : typing.Optional[int]
            H.264 CRF quality (0=lossless, 51=worst)

        max_duration : typing.Optional[float]
            Max recording duration in seconds

        width : typing.Optional[int]
            Video width in pixels (auto-detected from X11 if omitted)

        height : typing.Optional[int]
            Video height in pixels (auto-detected from X11 if omitted)

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ResponseDisplayRecordResult
            Successful Response

        Examples
        --------
        from agent_sandbox import Sandbox

        client = Sandbox(
            base_url="https://yourhost.com/path/to/api",
        )
        client.display.record(
            action="start",
        )
        """
        _response = self._raw_client.record(
            action=action,
            save_path=save_path,
            fps=fps,
            crf=crf,
            max_duration=max_duration,
            width=width,
            height=height,
            request_options=request_options,
        )
        return _response.data


class AsyncDisplayClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._raw_client = AsyncRawDisplayClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> AsyncRawDisplayClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        AsyncRawDisplayClient
        """
        return self._raw_client

    async def record(
        self,
        *,
        action: RecordAction,
        save_path: typing.Optional[str] = OMIT,
        fps: typing.Optional[int] = OMIT,
        crf: typing.Optional[int] = OMIT,
        max_duration: typing.Optional[float] = OMIT,
        width: typing.Optional[int] = OMIT,
        height: typing.Optional[int] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> ResponseDisplayRecordResult:
        """
        Control display screen recording (start/stop/status).

        Records the entire X11 desktop using ffmpeg x11grab,
        including browser UI, multiple tabs, popups, terminal, etc.

        Parameters
        ----------
        action : RecordAction
            Recording action: start, stop, or status

        save_path : typing.Optional[str]
            Output file path (default: /tmp/recordings/recording_{timestamp}.mp4)

        fps : typing.Optional[int]
            Frames per second

        crf : typing.Optional[int]
            H.264 CRF quality (0=lossless, 51=worst)

        max_duration : typing.Optional[float]
            Max recording duration in seconds

        width : typing.Optional[int]
            Video width in pixels (auto-detected from X11 if omitted)

        height : typing.Optional[int]
            Video height in pixels (auto-detected from X11 if omitted)

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ResponseDisplayRecordResult
            Successful Response

        Examples
        --------
        import asyncio

        from agent_sandbox import AsyncSandbox

        client = AsyncSandbox(
            base_url="https://yourhost.com/path/to/api",
        )


        async def main() -> None:
            await client.display.record(
                action="start",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.record(
            action=action,
            save_path=save_path,
            fps=fps,
            crf=crf,
            max_duration=max_duration,
            width=width,
            height=height,
            request_options=request_options,
        )
        return _response.data



# This file was auto-generated by Fern from our API Definition.

import typing
from json.decoder import JSONDecodeError

from ..core.api_error import ApiError
from ..core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ..core.http_response import AsyncHttpResponse, HttpResponse
from ..core.jsonable_encoder import jsonable_encoder
from ..core.pydantic_utilities import parse_obj_as
from ..core.request_options import RequestOptions
from ..errors.unprocessable_entity_error import UnprocessableEntityError
from ..types.http_validation_error import HttpValidationError
from ..types.response import Response
from ..types.response_list_sandbox_hook import ResponseListSandboxHook
from ..types.response_sandbox_hook import ResponseSandboxHook
from ..types.sandbox_response import SandboxResponse

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class RawSandboxClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper

    def get_context(self, *, request_options: typing.Optional[RequestOptions] = None) -> HttpResponse[SandboxResponse]:
        """
        Get sandbox environment information

        Parameters
        ----------
        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[SandboxResponse]
            Successful Response
        """
        _response = self._client_wrapper.httpx_client.request(
            "v1/sandbox",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    SandboxResponse,
                    parse_obj_as(
                        type_=SandboxResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def get_python_packages(self, *, request_options: typing.Optional[RequestOptions] = None) -> HttpResponse[Response]:
        """
        Get installed Python packages

        Parameters
        ----------
        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[Response]
            Successful Response
        """
        _response = self._client_wrapper.httpx_client.request(
            "v1/sandbox/packages/python",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    Response,
                    parse_obj_as(
                        type_=Response,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def get_nodejs_packages(self, *, request_options: typing.Optional[RequestOptions] = None) -> HttpResponse[Response]:
        """
        Get installed Node.js packages

        Parameters
        ----------
        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[Response]
            Successful Response
        """
        _response = self._client_wrapper.httpx_client.request(
            "v1/sandbox/packages/nodejs",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    Response,
                    parse_obj_as(
                        type_=Response,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def list_hooks(
        self, *, event: typing.Optional[str] = None, request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[ResponseListSandboxHook]:
        """
        List registered lifecycle hooks, optionally filtered by event.

        Parameters
        ----------
        event : typing.Optional[str]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[ResponseListSandboxHook]
            Successful Response
        """
        _response = self._client_wrapper.httpx_client.request(
            "v1/sandbox/hooks",
            method="GET",
            params={
                "event": event,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    ResponseListSandboxHook,
                    parse_obj_as(
                        type_=ResponseListSandboxHook,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        HttpValidationError,
                        parse_obj_as(
                            type_=HttpValidationError,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def register_hook(
        self,
        *,
        name: str,
        command: str,
        event: typing.Optional[str] = OMIT,
        timeout: typing.Optional[float] = OMIT,
        priority: typing.Optional[int] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[ResponseSandboxHook]:
        """
        Register a lifecycle hook. Currently supported events: shutdown.

        Parameters
        ----------
        name : str
            Unique name for this hook

        command : str
            Shell command to execute

        event : typing.Optional[str]
            Lifecycle event: "shutdown"

        timeout : typing.Optional[float]
            Per-hook timeout in seconds

        priority : typing.Optional[int]
            Execution priority (lower = earlier). Same priority hooks run in parallel

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[ResponseSandboxHook]
            Successful Response
        """
        _response = self._client_wrapper.httpx_client.request(
            "v1/sandbox/hooks",
            method="POST",
            json={
                "name": name,
                "event": event,
                "command": command,
                "timeout": timeout,
                "priority": priority,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    ResponseSandboxHook,
                    parse_obj_as(
                        type_=ResponseSandboxHook,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        HttpValidationError,
                        parse_obj_as(
                            type_=HttpValidationError,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def remove_hook(
        self, name: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[Response]:
        """
        Remove a hook by name. ENV hooks cannot be removed.

        Parameters
        ----------
        name : str

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[Response]
            Successful Response
        """
        _response = self._client_wrapper.httpx_client.request(
            f"v1/sandbox/hooks/{jsonable_encoder(name)}",
            method="DELETE",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    Response,
                    parse_obj_as(
                        type_=Response,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        HttpValidationError,
                        parse_obj_as(
                            type_=HttpValidationError,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)


class AsyncRawSandboxClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper

    async def get_context(
        self, *, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[SandboxResponse]:
        """
        Get sandbox environment information

        Parameters
        ----------
        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[SandboxResponse]
            Successful Response
        """
        _response = await self._client_wrapper.httpx_client.request(
            "v1/sandbox",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    SandboxResponse,
                    parse_obj_as(
                        type_=SandboxResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def get_python_packages(
        self, *, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[Response]:
        """
        Get installed Python packages

        Parameters
        ----------
        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[Response]
            Successful Response
        """
        _response = await self._client_wrapper.httpx_client.request(
            "v1/sandbox/packages/python",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    Response,
                    parse_obj_as(
                        type_=Response,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def get_nodejs_packages(
        self, *, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[Response]:
        """
        Get installed Node.js packages

        Parameters
        ----------
        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[Response]
            Successful Response
        """
        _response = await self._client_wrapper.httpx_client.request(
            "v1/sandbox/packages/nodejs",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    Response,
                    parse_obj_as(
                        type_=Response,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def list_hooks(
        self, *, event: typing.Optional[str] = None, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[ResponseListSandboxHook]:
        """
        List registered lifecycle hooks, optionally filtered by event.

        Parameters
        ----------
        event : typing.Optional[str]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[ResponseListSandboxHook]
            Successful Response
        """
        _response = await self._client_wrapper.httpx_client.request(
            "v1/sandbox/hooks",
            method="GET",
            params={
                "event": event,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    ResponseListSandboxHook,
                    parse_obj_as(
                        type_=ResponseListSandboxHook,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        HttpValidationError,
                        parse_obj_as(
                            type_=HttpValidationError,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def register_hook(
        self,
        *,
        name: str,
        command: str,
        event: typing.Optional[str] = OMIT,
        timeout: typing.Optional[float] = OMIT,
        priority: typing.Optional[int] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[ResponseSandboxHook]:
        """
        Register a lifecycle hook. Currently supported events: shutdown.

        Parameters
        ----------
        name : str
            Unique name for this hook

        command : str
            Shell command to execute

        event : typing.Optional[str]
            Lifecycle event: "shutdown"

        timeout : typing.Optional[float]
            Per-hook timeout in seconds

        priority : typing.Optional[int]
            Execution priority (lower = earlier). Same priority hooks run in parallel

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[ResponseSandboxHook]
            Successful Response
        """
        _response = await self._client_wrapper.httpx_client.request(
            "v1/sandbox/hooks",
            method="POST",
            json={
                "name": name,
                "event": event,
                "command": command,
                "timeout": timeout,
                "priority": priority,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    ResponseSandboxHook,
                    parse_obj_as(
                        type_=ResponseSandboxHook,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        HttpValidationError,
                        parse_obj_as(
                            type_=HttpValidationError,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def remove_hook(
        self, name: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[Response]:
        """
        Remove a hook by name. ENV hooks cannot be removed.

        Parameters
        ----------
        name : str

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[Response]
            Successful Response
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"v1/sandbox/hooks/{jsonable_encoder(name)}",
            method="DELETE",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    Response,
                    parse_obj_as(
                        type_=Response,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        HttpValidationError,
                        parse_obj_as(
                            type_=HttpValidationError,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)
